package com.pdaaphm.biz.service.impl;

import java.util.List;
import java.util.Map;

import com.pdaaphm.biz.domain.SubAlgorithm;
import com.pdaaphm.biz.dto.AlgorithmDTO;
import com.pdaaphm.biz.dto.SubAlgorithmDTO;
import com.pdaaphm.biz.mapper.AlgorithmIoFieldMapper;
import com.pdaaphm.biz.mapper.SubAlgorithmMapper;
import com.pdaaphm.common.utils.DateUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.pdaaphm.biz.mapper.AlgorithmMapper;
import com.pdaaphm.biz.domain.Algorithm;
import com.pdaaphm.biz.service.IAlgorithmService;

/**
 * 算法Service业务层处理
 *
 * @author xlk
 * @date 2023-07-26
 */
@Service
public class AlgorithmServiceImpl implements IAlgorithmService
{
    @Autowired
    private AlgorithmMapper algorithmMapper;
    @Autowired
    private SubAlgorithmMapper subAlgorithmMapper;
    @Autowired
    private AlgorithmIoFieldMapper algorithmIoFieldMapper;

    /**
     * 查询算法
     *
     * @param id 算法主键
     * @return 算法
     */
    @Override
    public Algorithm selectAlgorithmById(Long id)
    {
        return algorithmMapper.selectAlgorithmById(id);
    }

    /**
     * 查询算法对象
     *
     * @param id 算法主键
     * @return 算法
     */
    @Override
    public AlgorithmDTO getAlgorithmDto(Long id)
    {
        Algorithm algorithm = this.selectAlgorithmById(id);
        AlgorithmDTO algorithmDto = new AlgorithmDTO();
        List<SubAlgorithmDTO> l = algorithmIoFieldMapper.getIoSubList(algorithm.getSubTypeId(),id);
        BeanUtils.copyProperties(algorithm,algorithmDto);
        algorithmDto.setIoSubList(l);
        return algorithmDto;
    }

    /**
     * 查询算法列表
     *
     * @param algorithm 算法
     * @return 算法
     */
    @Override
    public List<Algorithm> selectAlgorithmList(Algorithm algorithm)
    {
        return algorithmMapper.selectAlgorithmList(algorithm);
    }

    /**
     * 新增算法
     *
     * @param algorithm 算法
     * @return 结果
     */
    @Override
    public int insertAlgorithm(Algorithm algorithm)
    {
        algorithm.setCreateTime(DateUtils.getNowDate());
        return algorithmMapper.insertAlgorithm(algorithm);
    }

    @Override
    public int addOrUpdateDto(AlgorithmDTO algorithmDto) {
        // todo dto 2 domain, save doman
        // get ioSubList, save SubAlgorithm
        int res;
        Algorithm algorithm = new Algorithm();
        BeanUtils.copyProperties(algorithmDto,algorithm);
        if(algorithmDto.getId() == null){
            res = this.insertAlgorithm(algorithm);
        } else {
            res = this.updateAlgorithm(algorithm);
        }
        List<SubAlgorithmDTO> ioSubList = algorithmDto.getIoSubList();
        SubAlgorithm subAlgorithm = new SubAlgorithm();
        for(int i = 0; i < ioSubList.size(); i++){
            SubAlgorithmDTO subAlgorithmDTO = ioSubList.get(i);
            BeanUtils.copyProperties(subAlgorithmDTO,subAlgorithm);
            subAlgorithm.setAlgorithmId(algorithm.getId());
            if(subAlgorithmDTO.getId() == null){
                subAlgorithmMapper.insertSubAlgorithm(subAlgorithm);
            } else {
                subAlgorithmMapper.updateSubAlgorithm(subAlgorithm);
            }
        }
        return res;
    }

    /**
     * 修改算法
     *
     * @param algorithm 算法
     * @return 结果
     */
    @Override
    public int updateAlgorithm(Algorithm algorithm)
    {
        algorithm.setUpdateTime(DateUtils.getNowDate());
        return algorithmMapper.updateAlgorithm(algorithm);
    }

    /**
     * 批量删除算法
     *
     * @param ids 需要删除的算法主键
     * @return 结果
     */
    @Override
    public int deleteAlgorithmByIds(Long[] ids)
    {
        return algorithmMapper.deleteAlgorithmByIds(ids);
    }

    /**
     * 删除算法信息
     *
     * @param id 算法主键
     * @return 结果
     */
    @Override
    public int deleteAlgorithmById(Long id)
    {
        return algorithmMapper.deleteAlgorithmById(id);
    }

    @Override
    public List<Map> getOption() { return algorithmMapper.getOption(); }
}
